@@ -9,6 +9,7 @@ Copyright (c) 2012 Multmeio [design+tecnologia]. All rights reserved.
 
 from django import forms
 from django.utils.safestring import mark_safe
+from django.template.defaultfilters import floatformat
 
 from hstore_flattenfields.utils import str2literal
 
@@ -18,15 +19,3 @@ class SelectMultipleWidget(forms.SelectMultiple):
         html = super(SelectMultipleWidget, self).render(
             name, str2literal(value), attrs, choices)
         return mark_safe(html)
-
-
-class RealCurrencyInput(forms.TextInput):
-    def render(self, name, value, attrs=None):
-        value = value or ''
-        attrs = attrs or {}
-
-        if isinstance(value, Decimal):
-            value = floatformat(str(value), 2)
-
-        html = super(RealCurrencyInput, self).render(name, value, attrs)
-        return mark_safe(html)
