@@ -13,29 +13,24 @@ used in hstore_flattenfields application.
 :license: BSD, see LICENSE for more details.
 """
 
-import sys
 from django.db import models
 from django.contrib.contenttypes.models import ContentType
 from django.utils.translation import ugettext as _
-from django.core.cache import cache
+
 from django_orm.postgresql import hstore
 from django_extensions.db.fields import AutoSlugField
 
 from db.base import (
     HStoreModel,
+    HStoreGroupedModel,
     HStoreM2MGroupedModel,
 )
 from hstore_flattenfields.utils import (
     single_list_to_tuple,
     FIELD_TYPES,
     FIELD_TYPES_WITHOUT_BLANK_OPTION,
-    all_flattenfields_tables_is_created,
-)
-from hstore_flattenfields.db.cache import (
-    DynamicFieldGroupCacheManager,
-    ContentPaneCacheManager,
-    DynamicFieldCacheManager
 )
+from hstore_flattenfields.signals import *
 
 
 # Setup the "class Meta:" flattenfields custom configs
@@ -66,34 +61,10 @@ class DynamicFieldGroup(models.Model):
     slug = AutoSlugField(populate_from='name', separator='_', max_length=100, unique=True, overwrite=True)
     description = models.TextField(null=True, blank=True, verbose_name=_('Description'))
 
-    # managers
-    objects = DynamicFieldGroupCacheManager(
-        cache_key="dynamic_field_groups",
-        prefetch_related = ['dynamic_fields', 'content_panes'],
-    )
-
     class Meta:
         verbose_name = _('Dynamic Field Group')
         verbose_name_plural = _('Dynamic Field Groups')
 
-    @property
-    def fields(self):
-        """
-        Returns all the related ``DynamicField`` instances from cache.
-
-        >>> group = DynamicFieldGroup.objects.create(name="Test Group")
-        >>> DynamicField.objects.create(refer="Something", group=group, name="something_age", verbose_name=u"Age")
-        <DynamicField: Age>
-        >>> group.fields
-        [<DynamicField: Age>]
-        """
-        dynamic_fields = cache.get('dynamic_fields', [])
-        def by_group(dynamic_field):
-            if hasattr(self, 'dynamicfieldgroup_ptr'):
-                return dynamic_field.group == self.dynamicfieldgroup_ptr
-            return dynamic_field.group == self
-        return filter(by_group, dynamic_fields)
-
     def __unicode__(self):
         """
         Returns a pretty representation of this object.
@@ -104,6 +75,20 @@ class DynamicFieldGroup(models.Model):
         """
         return u"%s" % self.name
 
+    @property
+    def fields(self):
+        """
+        Returns all the related ``DynamicField`` instances from cache.
+
+        >>> DynamicField.objects.all().delete()
+        >>> group = DynamicFieldGroup.objects.create(name="Test Group")
+        >>> DynamicField.objects.create(refer="Something", group=group, name="something_age", verbose_name=u"Age")
+        <DynamicField: Age>
+        >>> group.fields
+        [<DynamicField: Age>]
+        """
+        return DynamicField.objects.filter(models.Q(group=self) | models.Q(group=None))
+
 
 class ContentPane(models.Model):
     """
@@ -127,13 +112,6 @@ class ContentPane(models.Model):
     content_type = models.ForeignKey(ContentType, null=True, blank=True, related_name='content_panes')
     group = models.ForeignKey(DynamicFieldGroup, null=True, blank=True, related_name='content_panes', verbose_name=_("Groups"))
 
-    # managers
-    objects = ContentPaneCacheManager(
-        cache_key="content_panes",
-        prefetch_related = ['dynamic_fields'],
-        select_related = ['group', 'content_type']
-    )
-
     class Meta:
         verbose_name = _('Content Pane')
         verbose_name_plural = _('Content Panes')
@@ -162,7 +140,7 @@ class ContentPane(models.Model):
         >>> content_pane.fields
         [<DynamicField: Age>]
         """
-        return DynamicField.objects.cache_filter(cpane=self)
+        return self.dynamic_fields.order_by('order')
 
     @property
     def is_generic(self):
@@ -213,28 +191,14 @@ class DynamicField(models.Model):
     group = models.ForeignKey(DynamicFieldGroup, null=True, blank=True, related_name="dynamic_fields", verbose_name=_("Groups"))
     content_pane = models.ForeignKey(ContentPane, null=True, blank=True, related_name="dynamic_fields", verbose_name=_("Panel"))
 
-    # managers
-    objects = DynamicFieldCacheManager(
-        cache_key="dynamic_fields",
-        select_related=['content_pane', 'group']
-    )
-
     class Meta:
         verbose_name = _('Dynamic Field')
         verbose_name_plural = _('Dynamic Fields')
+        # ordering = ['pk']
 
     def __unicode__(self):
         return self.verbose_name or self.name
 
     @property
     def has_blank_option(self):
-        return self.blank and \
-            self.typo not in FIELD_TYPES_WITHOUT_BLANK_OPTION
-
-
-# NOTE: Skip the cache when we use the Test Mode
-flattenfields_models = [ContentPane, DynamicFieldGroup, DynamicField]
-if all_flattenfields_tables_is_created(flattenfields_models):
-    # Initial cache charge
-    for Model in flattenfields_models:
-        Model.objects.charge_cache()
+        return self.blank and self.typo not in FIELD_TYPES_WITHOUT_BLANK_OPTION
\ No newline at end of file
