@@ -9,16 +9,18 @@ The Models file where places all the stored classes
 used in hstore_flattenfields application.
 
 :copyright: 2012, multmeio (http://www.multmeio.com.br)
-:author: 2012, Luís Antônio Araújo Brito <iuridiniz@gmail.com>
+:author: 2012, Luís Araújo <caitifbrito@gmail.com>
 :license: BSD, see LICENSE for more details.
 """
 
 from django.core.exceptions import ValidationError, ImproperlyConfigured
 from django.db import connection
-from django.db.models import get_model
+from django.db.models import get_model, Q
+from django.contrib.contenttypes.models import ContentType
 from django.conf import settings
 from ast import literal_eval
 from datetime import datetime, date
+
 import re
 import six
 
@@ -26,6 +28,9 @@ import six
 DATETIME_ISO_RE = re.compile(
     r'(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2}) (?P<hour>\d{1,2}):(?P<minute>\d{1,2}):(?P<second>\d{1,2})$'
 )
+DATETIME_ISO_TZ_RE = re.compile(
+    r'(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2}) (?P<hour>\d{1,2}):(?P<minute>\d{1,2}):(?P<second>\d{1,2})$'
+)
 DATETIME_ISO_MS_RE = re.compile(
     r'(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2}) (?P<hour>\d{1,2}):(?P<minute>\d{1,2}):(?P<second>\d{1,2}).(?P<microsecond>\d{1,6})$'
 )
@@ -42,7 +47,7 @@ DATE_BR_RE = re.compile(
     r'(?P<day>\d{1,2})/(?P<month>\d{1,2})/(?P<year>\d{4})$'
 )
 
-REGEX_DATETIMES = [DATETIME_ISO_RE, DATETIME_ISO_MS_RE, DATETIME_BR_RE,
+REGEX_DATETIMES = [DATETIME_ISO_RE, DATETIME_ISO_TZ_RE, DATETIME_ISO_MS_RE, DATETIME_BR_RE,
                    DATETIME_BR_MS_RE]
 REGEX_DATES = [DATE_ISO_RE, DATE_BR_RE]
 
@@ -119,6 +124,7 @@ __all__ = ['single_list_to_tuple',
            # 'get_dynamic_field_model',
            'parse_queryset',
            'all_flattenfields_tables_is_created',
+           'get_ctype',
 ]
 
 
@@ -171,6 +177,7 @@ def str2datetime(value):
     if not value:
         return ''
 
+    # value = value.replace('T', ' ')
     for regex in REGEX_DATETIMES:
         match = regex.match(value)
         if match:
@@ -230,6 +237,12 @@ def has_any_in(chances, possibilities):
 
 
 def all_flattenfields_tables_is_created(Models):
+    """
+    >>> from hstore_flattenfields.utils import all_flattenfields_tables_is_created
+    >>> from hstore_flattenfields.models import *
+    >>> all_flattenfields_tables_is_created([DynamicField, ContentPane, DynamicFieldGroup])
+    True
+    """
     db_table_names = connection.introspection.table_names()
     flattenfields_tables = map(lambda x: x._meta.db_table, Models)
     return all([x in db_table_names for x in flattenfields_tables])
@@ -239,6 +252,11 @@ _DYNAMIC_FIELD_TABLE_EXISTS = None
 
 
 def dynamic_field_table_exists():
+    """
+    >>> from hstore_flattenfields.utils import dynamic_field_table_exists
+    >>> dynamic_field_table_exists()
+    True
+    """
     from hstore_flattenfields.models import DynamicField
     dynamic_field_table_name = DynamicField._meta.db_table
     global _DYNAMIC_FIELD_TABLE_EXISTS
@@ -248,17 +266,17 @@ def dynamic_field_table_exists():
 
 
 def get_fieldnames(fields, excludes=[]):
-    """
-    >>> from tests.app.models import Author
-    >>> get_fieldnames(Author._meta.fields)
-    ['id', '_dfields']
-    >>> from tests.app.models import Author
-    >>> get_fieldnames(Author._meta.fields, ['_dfields'])
-    ['id']
-    """
-    return map(lambda f: f.name,
-       filter(lambda f: f.name not in excludes, fields)
-   )
+    pass
+   #  """
+   #  >>> from tests.app.models import Author
+   #  >>> get_fieldnames(Author()._meta.fields)
+   #  ['id', '_dfields']
+   #  >>> get_fieldnames(Author()._meta.fields, ['_dfields'])
+   #  ['id']
+   #  """
+   #  return map(lambda f: f.name,
+   #     filter(lambda f: f.name not in excludes, fields)
+   # )
 
 
 def create_choices(choices=''):
@@ -351,11 +369,10 @@ def parse_queryset(model, result):
     <DynamicField: Age>
     >>> DynamicField.objects.create(refer="Author", name="birth_date", verbose_name=u"Birth Date", typo="Date")
     <DynamicField: Birth Date>
-    >>> Author.objects.create(age=20, birth_date='24/05/1992')
-    <Author: Author object>
-    >>> parse_queryset(Author, Author.objects.values('age'))
+    >>> author = Author.objects.create(age=20, birth_date='24/05/1992')
+    >>> parse_queryset(Author, Author.objects.filter(id=author.id).values('age'))
     [{'age': 20}]
-    >>> parse_queryset(Author, Author.objects.values('birth_date'))
+    >>> parse_queryset(Author, Author.objects.filter(id=author.id).values('birth_date'))
     [{'birth_date': datetime.date(1992, 5, 24)}]
     """
     dfield_names = model._meta.get_all_dynamic_field_names()
@@ -366,3 +383,15 @@ def parse_queryset(model, result):
                 item: field.to_python(res.get(item))
             })
     return result
+
+def get_ctype(model):
+    """
+    >>> from tests.app.models import Author
+    >>> get_ctype(Author)
+    <ContentType: author>
+    >>> get_ctype(None)
+    Traceback (most recent call last):
+      ...
+    AttributeError: 'NoneType' object has no attribute '_meta'
+    """
+    return ContentType.objects.get_for_model(model)
\ No newline at end of file
