@@ -23,6 +23,7 @@ class Something(HStoreGroupedModel):
 
 
 class Author(HStoreM2MGroupedModel):
+    # author_name = models.CharField(max_length=50)
     # relations
     author_groups = models.ManyToManyField(AuthorType, null=True, blank=True, related_name='authors', verbose_name=u'Group')
 
@@ -32,7 +33,19 @@ class Author(HStoreM2MGroupedModel):
 
     def __str__(self):
         if hasattr(self, 'author_name'):
-            return self.author_name
+            return "%s" % self.author_name
+        else:
+            return super(Author, self).__str__()
+
+
+class Illustrator(Author):
+    class Meta:
+        # hstore
+        hstore_related_field = 'author_groups'
+        
+    def __str__(self):
+        if hasattr(self, 'illustrator_name'):
+            return "%s" % self.illustrator_name
         else:
             return super(Author, self).__str__()
 
@@ -42,19 +55,22 @@ class Tag(HStoreModel):
 
     def __str__(self):
         if hasattr(self, 'tag_name'):
-            return self.tag_name
+            return "%s" % self.tag_name
         else:
             return super(Author, self).__str__()
 
 
 
 class Book(HStoreModel):
+    # pages = models.IntegerField()
+    
     author = models.ForeignKey('Author', null=True, blank=True)
+    illustrators = models.ManyToManyField('Illustrator', related_name='books')
     tags = models.ManyToManyField('Tag')
 
     def __str__(self):
         if hasattr(self, 'title'):
-            return self.title
+            return self.title or ''
         else:
             return "Book %s" % self.pk
 
@@ -62,7 +78,7 @@ class Book(HStoreModel):
 class Season(HStoreModel):
     def __str__(self):
         if hasattr(self, 'year'):
-            return self.year
+            return self.year or ''
         else:
             return super(Author, self).__str__()
 
