@@ -3,5 +3,4 @@ from test_queryset import *
 from test_model_fields import *
 from test_forms import *
 from test_dynamic_field_groups import *
-from test_inheritance import *
-
+from test_content_panes import *
\ No newline at end of file
