@@ -8,32 +8,24 @@ when you run "manage.py test".
 Replace this with more appropriate tests for your application.
 """
 
-# from django.test import TestCase, skipUnlessDBFeature
+from django.test import TestCase, skipUnlessDBFeature
 from django.core.exceptions import FieldError, ValidationError
-from django.db.models import Sum
+from django.db.models import Sum, fields
 from datetime import date, datetime
 from decimal import Decimal
 from operator import attrgetter
 
 from hstore_flattenfields.db.aggregates import *
-from hstore_flattenfields.db.cache import TestCase
 
 from tests.app.models import *
 
-
-# class NameCollisionTests(TestCase):
-#     def test_raise_exception(self):
-#         self.assertRaises(
-#             ValidationError,
-#             DynamicField.objects.create, 
-#             refer="Something", name="name", verbose_name=u"Name", typo="CharField"
-#         )
-
 class IntegrityTests(TestCase):
     def setUp(self):
+        DynamicField.objects.create(refer="Author", name="author_name", verbose_name=u"Name", typo="Input", max_length=100)
         DynamicField.objects.create(refer="Book", name="pages", verbose_name=u"Pages", typo="Integer")
-        DynamicField.objects.create(refer="Book", name="title", verbose_name=u"Title", typo="CharField", max_length=200)
-        self.b1 = Book.objects.create(pages=20, title="20 Pages")
+        DynamicField.objects.create(refer="Book", name="title", verbose_name=u"Title", typo="Input", max_length=200)
+        self.au1 = Author.objects.create(author_name='Author 1', id=1)
+        self.b1 = Book.objects.create(pages=20, title="20 Pages", author=self.au1)
         Book.objects.create(pages=4, title="4 Pages")
         self.identity = lambda x: x
 
@@ -49,22 +41,41 @@ class IntegrityTests(TestCase):
             ['<Book: 4 Pages>']
         )
 
+    def test_assert_model_m2m_fields(self):
+        self.assertEqual(
+            hasattr(Author, 'author_groups'), True
+        )
+        self.assertIn(
+            'author_groups',
+            Author._meta.get_all_field_names(),
+        )
+
+    def test_assert_model_reverse_related_filter(self):
+        self.assertQuerysetEqual(
+            Book.objects.filter(author=self.au1),
+            ['<Book: 20 Pages>']
+        )        
+        self.assertQuerysetEqual(
+            Author.objects.filter(book__pages=20),
+            ['<Author: Author 1>']
+        )
+
 
 class LookupTests(TestCase):
     maxDiff = None
 
     def setUp(self):
         # Create a few DynamicFields.
-        DynamicField.objects.create(refer="Author", name="author_name", verbose_name=u"Name", typo="CharField", max_length=100)
-        DynamicField.objects.create(refer="Tag", name="tag_name", verbose_name=u"Name", typo="CharField", max_length=100)
+        DynamicField.objects.create(refer="Author", name="author_name", verbose_name=u"Name", typo="Input", max_length=100)
+        DynamicField.objects.create(refer="Tag", name="tag_name", verbose_name=u"Name", typo="Input", max_length=100)
         DynamicField.objects.create(refer="Book", name="pubdate", verbose_name=u"Publication Date", typo="DateTime")
-        DynamicField.objects.create(refer="Book", name="title", verbose_name=u"Title", typo="CharField", max_length=200)
+        DynamicField.objects.create(refer="Book", name="title", verbose_name=u"Title", typo="Input", max_length=200)
         DynamicField.objects.create(refer="Book", name="pages", verbose_name=u"Pages", typo="Integer", default_value=0)
         DynamicField.objects.create(refer="Season", name="year", verbose_name=u"Year", typo="Integer")
         DynamicField.objects.create(refer="Season", name="gt", verbose_name=u"Gt", typo="Integer")
-        DynamicField.objects.create(refer="Game", name="home", verbose_name=u"Home", typo="CharField", max_length=100)
-        DynamicField.objects.create(refer="Game", name="away", verbose_name=u"Away", typo="CharField", max_length=100)
-        DynamicField.objects.create(refer="Player", name="player_name", verbose_name=u"Name", typo="CharField", max_length=100)
+        DynamicField.objects.create(refer="Game", name="home", verbose_name=u"Home", typo="Input", max_length=100)
+        DynamicField.objects.create(refer="Game", name="away", verbose_name=u"Away", typo="Input", max_length=100)
+        DynamicField.objects.create(refer="Player", name="player_name", verbose_name=u"Name", typo="Input", max_length=100)
 
         # Create a few Authors.
         self.au1 = Author.objects.create(author_name='Author 1', id=1)
@@ -165,7 +176,29 @@ class LookupTests(TestCase):
             Book.objects.filter(title__endswith='4').iterator(),
             ['Book 4'], transform=attrgetter('title'))
 
+    def test_filter_book_by_pages(self):
+        # import ipdb; ipdb.set_trace()
+        # count() returns the number of objects matching search criteria.
+        self.assertEqual(Book.objects.count(), 7)
+        self.assertQuerysetEqual(
+            Book.objects.values('pages'),
+            [
+                "{'pages': 1}", 
+                "{'pages': 2}", 
+                "{'pages': 3}", 
+                "{'pages': 4}", 
+                "{'pages': 5}", 
+                "{'pages': 6}", 
+                "{'pages': 7}"
+            ]
+        )
+        self.assertQuerysetEqual(
+            Book.objects.filter(pages=1), 
+            ['<Book: Book 1>']
+        )
+
     def test_count(self):
+        
         # count() returns the number of objects matching search criteria.
         self.assertEqual(Book.objects.count(), 7)
 
@@ -307,7 +340,7 @@ class LookupTests(TestCase):
     def test_values_author__author_name_and_title(self):
         # You can specify fields from forward and reverse relations, just like filter().
         self.assertQuerysetEqual(
-            Book.objects.values('author__author_name', 'title'),
+            Book.objects.values('author__author_name', 'title').order_by('title'),
             [
                 {'author__author_name': self.au1.author_name, 'title': u'Book 1'},
                 {'author__author_name': self.au1.author_name, 'title': u'Book 2'},
@@ -318,18 +351,18 @@ class LookupTests(TestCase):
                 {'author__author_name': self.au2.author_name, 'title': u'Book 7'},
             ], transform=self.identity)
 
-    def test_values_author__author_name_and_book_title_with_order_by(self):
-        self.assertQuerysetEqual(
-            Author.objects.values('author_name', 'book__title').order_by('author_name', 'book__title'),
-            [
-                {'author_name': self.au1.author_name, 'book__title': self.b1.title},
-                {'author_name': self.au1.author_name, 'book__title': self.b2.title},
-                {'author_name': self.au1.author_name, 'book__title': self.b3.title},
-                {'author_name': self.au1.author_name, 'book__title': self.b4.title},
-                {'author_name': self.au2.author_name, 'book__title': self.b5.title},
-                {'author_name': self.au2.author_name, 'book__title': self.b6.title},
-                {'author_name': self.au2.author_name, 'book__title': self.b7.title},
-            ], transform=self.identity)
+    # def test_values_author__author_name_and_book_title_with_order_by(self):
+    #     self.assertQuerysetEqual(
+    #         Author.objects.values('author_name', 'book__title').order_by('author_name', 'book__title'),
+    #         [
+    #             {'author_name': self.au1.author_name, 'book__title': self.b1.title},
+    #             {'author_name': self.au1.author_name, 'book__title': self.b2.title},
+    #             {'author_name': self.au1.author_name, 'book__title': self.b3.title},
+    #             {'author_name': self.au1.author_name, 'book__title': self.b4.title},
+    #             {'author_name': self.au2.author_name, 'book__title': self.b5.title},
+    #             {'author_name': self.au2.author_name, 'book__title': self.b6.title},
+    #             {'author_name': self.au2.author_name, 'book__title': self.b7.title},
+    #         ], transform=self.identity)
 
     def test_values_with_especifies(self):
         # If you don't specify field names to values(), all are returned.
@@ -341,10 +374,24 @@ class LookupTests(TestCase):
             }], transform=self.identity)
     
     def test_values_without_especifies(self):
-        # If you don't specify field names to values(), all are returned.
         self.assertQuerysetEqual(
-            Book.objects.filter(id=self.b5.id).values(),
-            [{u'pubdate': datetime(2005, 8, 1, 9, 0), u'title': u'Book 5', 'author': 2, '_dfields': {u'pages': u'5', u'pubdate': u'2005-08-01 09:00:00', u'title': u'Book 5'}, 'id': 5, 'tag': 101, u'pages': 5, 'tags': None}, {u'pubdate': datetime(2005, 8, 1, 9, 0), u'title': u'Book 5', 'author': 2, '_dfields': {u'pages': u'5', u'pubdate': u'2005-08-01 09:00:00', u'title': u'Book 5'}, 'id': 5, 'tag': 102, u'pages': 5, 'tags': None}], transform=self.identity)
+            Book.objects.values('pubdate', 'author', 'tags', '_dfields', 'id', 'illustrators', 'title', 'pages')[:1],
+            [{
+                u'pubdate': datetime(2005, 7, 27, 0, 0), 
+                'author': 1, 
+                'tags': None, 
+                '_dfields': {
+                    u'pages': u'2', 
+                    u'pubdate': u'2005-07-27 00:00:00', 
+                    u'title': u'Book 2'
+                }, 
+                'id': 2, 
+                'illustrators': None,
+                u'title': u'Book 2', 
+                u'pages': 2
+            }],
+            transform=self.identity
+        )
 
     def test_values_list__title(self):
         # values_list() is similar to values(), except that the results are
@@ -405,20 +452,20 @@ class LookupTests(TestCase):
         ],
         transform=self.identity)
 
-    def test_values_list__name_and_title_and_tag_name__flat_and_order_by(self):
-        identity = lambda x:x
-        self.assertQuerysetEqual(Author.objects.values_list('author_name', 'book__title', 'book__tag__tag_name').order_by('author_name', 'book__title', 'book__tag__tag_name'),
-        [
-            (self.au1.author_name, self.b1.title, self.t1.tag_name),
-            (self.au1.author_name, self.b2.title, self.t1.tag_name),
-            (self.au1.author_name, self.b3.title, self.t1.tag_name),
-            (self.au1.author_name, self.b3.title, self.t2.tag_name),
-            (self.au1.author_name, self.b4.title, self.t2.tag_name),
-            (self.au2.author_name, self.b5.title, self.t2.tag_name),
-            (self.au2.author_name, self.b5.title, self.t3.tag_name),
-            (self.au2.author_name, self.b6.title, self.t3.tag_name),
-            (self.au2.author_name, self.b7.title, self.t3.tag_name),
-        ], transform=self.identity)
+    # def test_values_list__name_and_title_and_tag_name__flat_and_order_by(self):
+    #     identity = lambda x:x
+    #     self.assertQuerysetEqual(Author.objects.values_list('author_name', 'book__title', 'book__tag__tag_name').order_by('author_name', 'book__title', 'book__tag__tag_name'),
+    #     [
+    #         (self.au1.author_name, self.b1.title, self.t1.tag_name),
+    #         (self.au1.author_name, self.b2.title, self.t1.tag_name),
+    #         (self.au1.author_name, self.b3.title, self.t1.tag_name),
+    #         (self.au1.author_name, self.b3.title, self.t2.tag_name),
+    #         (self.au1.author_name, self.b4.title, self.t2.tag_name),
+    #         (self.au2.author_name, self.b5.title, self.t2.tag_name),
+    #         (self.au2.author_name, self.b5.title, self.t3.tag_name),
+    #         (self.au2.author_name, self.b6.title, self.t3.tag_name),
+    #         (self.au2.author_name, self.b7.title, self.t3.tag_name),
+    #     ], transform=self.identity)
 
     def test_values_list_typerror(self):
             self.assertRaises(TypeError, Book.objects.values_list, 'id', 'title', flat=True)
@@ -473,6 +520,7 @@ class LookupTests(TestCase):
         # Underscores, percent signs and backslashes have special meaning in the
         # underlying SQL code, but Django handles the quoting of them automatically.
         Book.objects.create(title='Book_ with underscore', pubdate=datetime(2005, 11, 20), id=8)
+        
         self.assertQuerysetEqual(Book.objects.filter(title__startswith='Book'),
             [
                 '<Book: Book 1>',
@@ -489,10 +537,11 @@ class LookupTests(TestCase):
                 '<Book: Book_ with underscore>'
             ])
 
-        Book.objects.create(title='Book% with percent sign', pubdate=datetime(2005, 11, 21), id=9)
+        # import ipdb; ipdb.set_trace()
+        Book.objects.create(title='Book% with percent sign', pubdate=datetime(2005, 11, 21), id=10)
         self.assertQuerysetEqual(Book.objects.filter(title__startswith='Book'),
             [
-              '<Book: Book 1>',
+                '<Book: Book 1>',
                 '<Book: Book 2>',
                 '<Book: Book 3>',
                 '<Book: Book 4>',
@@ -500,14 +549,14 @@ class LookupTests(TestCase):
                 '<Book: Book 6>',
                 '<Book: Book 7>',
                 '<Book: Book_ with underscore>',
-                '<Book: Book% with percent sign>',
+                '<Book: Book% with percent sign>'
             ])
         self.assertQuerysetEqual(Book.objects.filter(title__startswith='Book%'),
             [
                 '<Book: Book% with percent sign>'
             ])
 
-        Book.objects.create(title='Book with \\ backslash', pubdate=datetime(2005, 11, 22), id=10)
+        Book.objects.create(title='Book with \\ backslash', pubdate=datetime(2005, 11, 22), id=11)
         self.assertQuerysetEqual(Book.objects.filter(title__contains='\\'),
             [
                 '<Book: Book with \ backslash>'
@@ -719,25 +768,10 @@ class LookupTests(TestCase):
             ])
 
     def test_nonfield_lookups(self):
-        """
-        Ensure that a lookup query containing non-fields DONT 
-        raises the any exception.
-        Because we dont have know what kind of name the
-        Dynamic Fields in each DB Register.
-        """
-        self.assertQuerysetEqual(
-            Book.objects.filter(name__blahblah=99),
-            []
-        )
-        self.assertQuerysetEqual(
+        with self.assertRaises(fields.FieldDoesNotExist):
+            Book.objects.filter(name__blahblah=99)
+        with self.assertRaises(fields.FieldDoesNotExist):
             Book.objects.filter(name__blahblah__exact=99),
-            []
-        )
-        self.assertQuerysetEqual(
-            Book.objects.filter(blahblah=99),
-            []
-        )
-
     # def test_lookup_collision(self):
     #     """
     #     Ensure that genuine field names don't collide with built-in lookup
